#
# Copyright (c) 2012  Western Digital Technologies, Inc.  All rights reserved.
#

param($act,$sn)

function PerformAction([string]$action, [string]$serviceName)
{
    #
    # get service, terminate if cannot
    # note: return values below 25 used by 
    # win32_service StopService & StartService
    #
    
    $result = 0
    switch ($action)
    {
        "start" { $result = StartSC $serviceName }
        "stop"  { $result = StopSC  $serviceName }
        "query" { $result = QuerySC $serviceName }
        default { $result = 28 }
    }

    return $result
}

function GetService([string]$serviceName)
{
    $service = gwmi win32_service -filter "Name='$serviceName'"
    if ($service -eq $null) { Write-Host "Service" $serviceName "not found!" }

    return $service;
}

function WaitFor([string]$serviceName, [string]$svcState)
{
    $waitSecs = 2
    $maxWaits = 3
    $att = 1
        
    $service = GetService $serviceName
    if ($service -eq $null) { return 25 }
    
    Write-Host "Verifying" $serviceName "is" $svcState ". Current state is" $service.State
    while (($service.State -ne $svcState) -and ($att -le $maxWaits))
    {
        Write-Host "Waiting" $waitSecs "seconds for" $service.Name "to be" $svcState "(" $att "of" $maxWaits ")..."
        Start-Sleep -s $waitSecs
        $att += 1
        
        $service = GetService $serviceName
        if ($service -eq $null) { return 25 }
    } 
}

function StartService([string]$serviceName)
{
    Write-Host "Starting service" $serviceName "..."
    
    $service = GetService $serviceName
    if ($service -eq $null) { return 25 }
    
    $result = $service.StartService().ReturnValue
    switch ($result)
    {
        0       { 
                    WaitFor $serviceName "Running"
                    Write-Host "Service" $serviceName "started." 
                }
        default { Write-Host "Win32_Service StartService failed. Error" $result }
    }
    return $result
}

function StartSC([string]$serviceName)
{
    $maxAttempts = 3

    for ($attempt=1; $attempt -le $maxAttempts; $attempt++)
    {
        $service = GetService $serviceName
        if ($service -eq $null) { return 25 }
    
        Write-Host "Service" $service.Name "state is" $service.State
        switch ($service.State)
        {
            "Stopped"
                {
                    $startresult = StartService $serviceName
                    return $startresult
                }
            "Stop Pending"
                { 
                    Write-Host "Waiting 10 seconds for" $serviceName "to be stopped (" $attempt "of" $maxAttempts ")..." 
                    Start-Sleep -s 10 
                }
            "Running"
                { return 0 }
            "Start Pending"
                { return 0 }
            default
                { return 26 }
        }
    }
    return 27
}

function StopService([string]$serviceName)
{
    Write-Host "Stopping service" $serviceName "..."
    
    $service = GetService $serviceName
    if ($service -eq $null) { return 25 }
    
    $result = $service.StopService().ReturnValue
    switch ($result)
    {
        0       {
                    WaitFor $serviceName "Stopped"
                    Write-Host "Service" $serviceName "stopped."
                }
        default { Write-Host "Win32_Service StopService failed. Error $result." }
    }
    return $result
}

function StopSC([string]$serviceName)
{
    $maxAttempts = 3

    for ($attempt=1; $attempt -le $maxAttempts; $attempt++)
    {
        $service = GetService $serviceName
        if ($service -eq $null) { return 25 }
    
        Write-Host "Service " $service.Name "state is" $service.State
        switch ($service.State)
        {
            "Running"
                { 
                    $stopresult = StopService $serviceName
                    return $stopresult
                }
            "Start Pending"
                {   
                    Write-Host "Waiting 10 seconds for"$service.Name"to be running ("$attempt " of " $maxAttempts ")..."
                    Start-Sleep -s 10
                }
            "Stopped"
                { return 0 }
            "Stop Pending"
                { return 0 }
            default
                { return 26 }
        }
    }
    return 27
}

function QuerySC([string]$serviceName)
{
    $service = GetService $serviceName
    if ($service -eq $null) { return 25 }
    
    Write-Host $service.DisplayName"Service  ("$service.Name") details:"
    Write-Host "   Description: "$service.Description
    Write-Host "   Caption:     "$service.Caption
    Write-Host "   ProcessId:   "$service.ProcessId
    Write-Host "   ServiceType: "$service.ServiceType
    Write-Host "   Started:     "$service.Started
    Write-Host "   StartMode:   "$service.StartMode
    Write-Host "   State:       "$service.State
    Write-Host "   Status:      "$service.Status
    return 0
}

function ExitWithCode 
{ 
    param ( $exitcode )
    $host.SetShouldExit($exitcode) 
    exit 
}

$retval = PerformAction $act $sn
# ExitWithCode $retval
exit $retval
